import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:max_flutter_engine/engine_config.dart';
import 'package:max_flutter_engine/generated/l10n.dart';
import 'package:package_info/package_info.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';

import 'verson_info.dart';

typedef MRatingCheckerCallback = void Function(bool);

class AppStoreVersionChecker {
  factory AppStoreVersionChecker() => _sharedInstance();
  static AppStoreVersionChecker _instance;

  AppStoreVersionChecker._();

  static AppStoreVersionChecker _sharedInstance() {
    if (_instance == null) {
      _instance = AppStoreVersionChecker._();
    }
    return _instance;
  }

  bool _isProgressing = false;

  static const _lastCheckDate = "verson_check_date";

  autoCheckForUpdates(BuildContext context) async {
    if (await _hasCheckedToday()) {
      return;
    }
    await checkForUpdates(context, autoCheck: true);
  }

  checkForUpdates(BuildContext context, {autoCheck = false}) async {
    // TODO: android 版本检测
    if (Platform.isAndroid) return;
    if (_isProgressing) return;

    _isProgressing = true;
    var res = await http.get(Uri.encodeFull(EngineConfig.iosLookupURL));
    if (res.statusCode == HttpStatus.ok) {
      final VersionInfo versionInfo = VersionInfo.fromJson(json.decode(res.body));
      if (versionInfo != null && versionInfo.resultCount != null && versionInfo.resultCount > 0) {
        final VersionResult versionResult = versionInfo.results.first;
        PackageInfo packageInfo = await PackageInfo.fromPlatform();
        if (versionResult.version != null) {
          _markChecked();
          if (_hasNew(versionResult, packageInfo)) {
            if (autoCheck) {
              _showCurrentNew(versionResult.version, context);
            } else {
              _update();
            }
          } else {
            if (!autoCheck) {
              _showNewAlert(packageInfo.version, context);
            }
          }
          return;
        }
      }
      debugPrint("${res.body}");
      if (!autoCheck) {
        _showDataErrorAlert(context);
      }
    } else {
      if (!autoCheck) {
        _showNetErrorAlert(context);
      }
    }
    _isProgressing = false;
  }

  _hasCheckedToday() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String result = prefs.getString(_lastCheckDate) == null ? "" : prefs.getString(_lastCheckDate);
    return _today() == result;
  }

  _markChecked() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setString(_lastCheckDate, _today());
  }

  _today() {
    var now = new DateTime.now();
    return "${now.year}-${now.month}-${now.day}";
  }

  _hasNew(VersionResult versonResult, PackageInfo packageInfo) {
    return versonResult.version.compareTo(packageInfo.version) == 1;
  }

  _showCurrentNew(String version, BuildContext context) async {
    showDialog(
        context: context,
        builder: (_) => AlertDialog(
                title: Text(ES.of(context).version_updated),
                content: Text(ES.of(context).new_version_version_is_ready_for_you(version)),
                actions: <Widget>[
                  FlatButton(
                    child: Text(ES.of(context).not_now),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  ),
                  FlatButton(
                    child: Text(ES.of(context).update_now),
                    onPressed: () {
                      Navigator.of(context).pop();
                      _update();
                    },
                  )
                ]));
  }

  _showNewAlert(String version, BuildContext context) async {
    showDialog(
        context: context,
        builder: (_) => AlertDialog(
                title: Text(ES.of(context).you_are_using_the_latest_version),
                content: Text(ES.of(context).version_is_the_current_version(version)),
                actions: <Widget>[
                  FlatButton(
                    child: Text(ES.of(context).ok),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  )
                ]));
  }

  _showDataErrorAlert(BuildContext context) async {
    showDialog(
      context: context,
      builder: (_) => AlertDialog(
        title: Text(ES.of(context).sorry_the_operation_failed),
        content: Text(ES.of(context).data_parsing_error_please_try_again_later),
        actions: <Widget>[
          FlatButton(
            child: Text(ES.of(context).ok),
            onPressed: () {
              Navigator.of(context).pop();
            },
          ),
        ],
      ),
    );
  }

  _showNetErrorAlert(BuildContext context) async {
    showDialog(
        context: context,
        builder: (_) => AlertDialog(
                title: Text(ES.of(context).sorry_the_operation_failed),
                content: Text(ES.of(context).network_connection_is_unstable_please_try_again_later),
                actions: <Widget>[
                  FlatButton(
                    child: Text(ES.of(context).ok),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  )
                ]));
  }

  _update() {
    launch(EngineConfig.iosUpdateURL);
  }
}
